//  Objectif :
//	Dfinition d'un point mathmatique dans un espace plan ( deux
//	dimmensions) muni d'un repre cartsien
//

#ifndef Point__H
#define Point__H

#include <iostream>

class Point {
public:		// Constructeur

    Point(double vx, double vy);
	// Constructeur  partir des coordonnes cartsiennes vx et vy.
	//
	//@ ensures x() == vx;
	//@ ensures y() == vy;

public: 	// Vue cartsienne

    double x() const;
	// Abscisse du point.

    double y() const;
	// Ordonne du point.

    void set_x(double nx);
	// Changer l'abscisse du point.
	//
	//@ ensures x() == nx;

    void set_y(double ny);
	// Changer l'ordonne du point.
	//
	//@ ensures y() == ny;

    void set_cartesien(double vx, double vy);
	// Changer les coordonnes cartsiennes du point.
	//
	//@ ensures x() == vx;	// x initialis
	//@ ensures y() == vy;	// y initialis

public:		// Autres requtes

    double distance(const Point &p) const;
	// Distance du point  un autre.

public:		// galit et ingalit

    bool operator == (const Point & autre) const;
	// galit de deux points
	//
	//@ ensures Result implies (x() == autre.x()) && (y() == autre.y());

    bool operator != (const Point & autre) const;
	// non galit de deux points
	//
	//@ ensures Result == !(*this == autre);

public:		// surcharge des oprateurs arithmtiques

    Point operator + (const Point &p2) const;
	// opration interne
	//
	//@ ensures \Result.x() == x() + p2.x();
	//@ ensures \Result.y() == y() + p2.y();

    Point& operator += (const Point &p2);
	// opration interne
	//
	//@ ensures x() == \Old(x()) + p2.x();
	//@ ensures y() == \Old(y()) + p2.y();

    Point operator * (double r) const;
	// produit externe
	//
	//@ ensures \result.x() == x() * r;
	//@ ensures \result.y() == y() * r;

    double operator * (const Point &p2) const;
	// produit scalaire
	//
	//@ ensures \result == x() * p2.x() + y() * p2.y();

public:		// Autres commandes

    void translater(double dx, double dy);
	// Translater le point de dx suivant l'axe des X et dy suivant l'axe
	// des Y
	//
	//@ ensures x() == \Old(x()) + dx;
	//@ ensures y() == \Old(y()) + dy;

    void afficher(std::ostream &out = std::cout) const;
	// Afficher le point


public:		// Pour tracer la cration/destruction des des points

    ~Point();
	// Destructeur

    Point(const Point &autre);
	// Constructeur de copie
	//
	//@ ensures *this == autre;	// copi

    Point & operator = (const Point & autre);
	// Oprateur d'affectation
	//
	//@ ensures *this == autre;	// copi


private:
    double _x;	// Abscisse du point
    double _y;	// Ordonne du point

};

// Surcharges d'oprateurs

Point operator * (double facteur, const Point & p);

std::ostream & operator << (std::ostream & out, const Point &p);

#endif
